// Prosta gra w zapamiętywanie kolorów
// - program dla Arduino
// Autor: Tomasz Rękawek
// http://newton.net.pl/

int latchPin = 8, clockPin = 12, dataPin = 3; //rejestr przesuwający
byte data = 0; //dane do wysłania na rejestr

int led8 = 7, led9 = 6; //dodatkowe LEDy
int ledR = 11, ledG = 10, ledB = 9; //LED RGB
int buttonLeft = 5, buttonRight = 4; //przyciski
int speakerPin = 13; //głośniczek

void setup()
{
  Serial.begin(57600);
  for(int i=3;i<=13;i++) // ustawienie wszystkich pinów
    pinMode(i, OUTPUT); // od 3 do 13 jako wyjścia
  pinMode(buttonLeft, INPUT); // ustawienie pinów przycisków
  pinMode(buttonRight, INPUT); // jako wejścia
  
  digitalWrite(buttonLeft, HIGH); // włączenie wewnętrznych
  digitalWrite(buttonRight, HIGH); // rezystorów podciągających
  randomSeed(analogRead(0)); // zainicjalizowanie generatora liczb pseudolosowych
  // wartością odczytaną z niepodłączonego analogowego pinu 0 - przy każdym 
  // uruchomieniu będzie ona inna
  clearLeds();
}

// główna pętla
void loop()
{
  effect();
  clearLeds();
  if(waitForButton() == 3)
    return;
  
  for(int j=3;j<=8;j++)
    if(!playLevel(7, j))
      return;
}

// play one level
// colors - ilość kolorów możliwych do wyboru
// howMuch - ilość kolorów do zapamiętania
boolean playLevel(int colors, int howMuch)
{
  byte level[8];
  boolean ok = true;
  int btn, color;
  
  for(int i=0;i<howMuch;i++)
    level[i] = random(colors);
  showLevel(level, howMuch);
  clearLeds();
  
  for(int i=0;i<howMuch;i++)
  {
    color = -1;
    setOne(i);
    while(true)
    {
      btn = waitForButton();
      // oba przyciski - restart gry
      if(btn == 3)
        return false;
      // użytkownik chce obejrzeć sekwencję jeszcze raz
      if(btn == 2 && i == 0 && color == -1)
      {
        showLevel(level, howMuch);
        clearLeds();
        setOne(i);
        continue;
      }
      // wybiera kolor
      else if(btn == 1)
      {
        color += 1;
        color %= colors;
        setColor(color);
        playTone(color * 100 + 900, 200); // częstotliwość odgrywanego dźwięku zależy od numeru koloru
      }
      // chce przejść do następnej pozycji
      // ale nie wybrał koloru
      else if(btn == 2 && color == -1)
        continue;
      // przechodzi do następnej pozycji
      else
      {
        playTone(color * 100 + 900, 500);
        if(level[i] != color)
          ok = false;
        break;
      }
    }
  }
  set(ok ? 8 : 9, true);
  if(waitForButton() == 3)
    return false;
  clearLeds();
  return ok;
}

// pokazuje poziom do zapamiętania
void showLevel(byte *level, int s)
{
  for(int i=0;i<s;i++)
  {
    setOne(i);
    setColor(level[i]);
    playTone(level[i] * 100 + 900, 1000);
    delay(50);
  } 
}

// efekty świetlne i dźwiękowe
void effect()
{
  for(int i=0;i<8;i++)
  {
    data = 0;
    set(i, true);
    setColor(i);
    playTone(i * 100 + 900, 50);
  }
  for(int i=7;i>=0;i--)
  {
    data = 0;
    set(i, true);
    setColor(i);
    playTone(i * 100 + 900, 50);
  }
}

// wybranie koloru diody RGB
void setColor(int color)
{
  switch(color + 1)
  {
    case 0: //nic
    setRGB(0, 0, 0);
    break;
    
    case 1: //niebieski
    setRGB(0, 0, 128);
    break;
    
    case 2: //zielony
    setRGB(0, 128, 0);
    break;
    
    case 3: //czerwony
    setRGB(255, 0, 0);
    break;
    
    case 4: //seledynowy
    setRGB(0, 128, 128);
    break;
    
    case 5: //fioletowy
    setRGB(32, 0, 64);
    break;
    
    case 6: //żółty
    setRGB(64, 128, 0);
    break;
    
    case 7: //biały
    setRGB(128, 255, 255);
    break;
  }
}

// wybranie wartości koloru diody RGB
void setRGB(int r, int g, int b)
{
  analogWrite(ledR, r);
  analogWrite(ledG, g);
  analogWrite(ledB, b);
}

// wygaszenie wszystkich diod
void clearLeds()
{
  data = 0;
  set(0, false);
  set(8, false);
  set(9, false);
  setRGB(0,0,0);
}

// zapalenie jednej diody i wygaszenie pozostałych
void setOne(int i)
{
  clearLeds();
  set(i, true);
}

// zapalenie lub wygaszenie jednej diody
void set(int i, boolean s)
{
  if(i == 8)
    digitalWrite(led8, s ? HIGH : LOW);
  if(i == 9)
    digitalWrite(led9, s ? HIGH : LOW);
  else
  {
    digitalWrite(latchPin, LOW);
    if(s)
      data = data | (1 << i);
    else
      data = data & (255 - 1 << i);
    shiftOut(dataPin, clockPin, MSBFIRST, data);
    digitalWrite(latchPin, HIGH);
  }
}

// sprawdzenie stanu przycisku i (1 dla lewego lub 2 dla prawego)
boolean get(int i)
{
  switch(i)
  {
    case 1:
    return digitalRead(buttonLeft) == LOW;
    
    case 2:
    return digitalRead(buttonRight) == LOW;
    
    default:
    return false;
  }
}

// czekaj na wciśnięcie przycisku i zwróć jego numer
// 1 - lewy
// 2 - prawy
// 3 - oba
int waitForButton()
{
  int b = 0;
  while(!b)
  {
    if(get(1))
      b = 1;
    if(get(2))
      b = 2;
  }
  while(get(b))
    if(get(b == 1 ? 2 : 1))
      b = 3;
  delay(50);
  return b;
}

// generuj dźwięk na głośniczku
void playTone(int tone, int duration)
{
  for (long i = 0; i < duration * 1000L; i += tone * 2)
  {
    digitalWrite(speakerPin, HIGH);
    delayMicroseconds(tone);
    digitalWrite(speakerPin, LOW);
    delayMicroseconds(tone);
  }
}
